#pragma comment(linker,"/STACK:256000000")

#ifdef __GNUC__
#define int64 long long
#else /* MSVC, say */
#define int64 __int64
#endif 

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <memory.h>
#include <ctype.h>
#include <stdlib.h>
#include <iostream>
#include <sstream>
#include <vector>
#include <string>
#include <algorithm>
#include <map>
#include <set>
#include <queue>
#include <stack>
#include <assert.h>

using namespace std;

#define MAX(a,b) (((a) > (b)) ? (a) : (b))
#define MIN(a,b) (((a) < (b)) ? (a) : (b))



int n, m, outs;

struct gate {
	int inputs[2];
	int type;
	int outvalue;
	
	void getoutvalue(int i0, int i1) {
		if (type & (1 << (i0 * 2 + i1))) {
			outvalue = 1;
		} else {
			outvalue = 0;
		}
	}
};

#define MAXMASK (1 << 16)

vector< gate > gates;
vector< vector< int > > values;
vector< int > out;
vector<int> outtable[MAXMASK];

void printtextableh() {
	freopen("schemetableh.tex", "w", stdout);
	printf("\\documentclass{article}\n");
	printf("\\begin{document}\n");
	printf("\\begin{center}\n");
	printf("\\begin{tabular}");
		
	printf("{");
	for (int i = 0; i < (1 << n) + 1; i++) {
		printf("|c");
	}
	printf("|");
	printf("}\n");
	printf("\\hline\n");
	
	for (int i = 1; i <= n + m + outs; i++) {
		if (i <= n - outs) {
			printf("$x_{k+%d}$ & ", i);
		} else 	if ((n - outs < i) && (i <= n)) {
			printf("$z_%d$ & ", i - (n - outs));
		} else if ((n < i) && (i <= n + m)) {
			printf("$g_%d$ & ", i - n);
		} else if (n + m < i) {
			printf("$z_%d'$ & ", i - (n + m));
		}
		for (int mask = 0; mask < (1 << n); mask++) {
			if (i <= n + m) {
				printf("%d ", outtable[mask][i]);			
			} else {
				int curout = outs - (i - (n + m));
				printf("%d ", outtable[mask][out[curout]]);
			}
		}
		printf("\\\\");
	}
	printf("\\hline\n");
	
	printf("\\end{tabular}\n");
	printf("\\end{center}\n");
	printf("\\end{document}\n");
}

void printtextablev() {
	freopen("schemetablev.tex", "w", stdout);
	printf("\\documentclass{article}\n");
	printf("\\begin{document}\n");
	printf("\\begin{center}\n");
	printf("\\begin{tabular}");
	printf("{");
	for (int i = 0; i < n + m + outs; i++) {
		printf("|c");
	}
	printf("|");
	printf("}\n");
	printf("\\hline\n");
	for (int i = 1; i <= n - outs; i++) {
		printf("$x_{k+%d}$ & ", i);
	}
	for (int i = 1; i <= outs; i++) {
		printf("$z_%d$ & ", i);
	}
	for (int i = 1; i <= m; i++) {
		printf("$g_%d$ & ", i);
	}
	for (int i = 1; i <= outs; i++) {
		printf("$z_%d'$ ", i);
		if (i != outs) {
			printf("& ");
		}
	}
	printf("\\\\");
	printf("\\hline\n");
	for (int mask = 0; mask < (1 << n); mask++) {
		for (int i = 1; i <= n + m; i++) {
			printf("%d & ", outtable[mask][i]);
		}
		for (int i = 0; i < outs; i++) {
			printf("%d ", outtable[mask][out[outs - i - 1]]);		
			if (i + 1 != outs) {
				printf("& ");	
			}
		}
		printf("\\\\");
	}
	printf("\\hline\n");
	printf("\\end{tabular}\n");
	printf("\\end{center}\n");
	printf("\\end{document}\n");
}



int main(int argc, char** argv) {
	if (argc != 3) {
		printf("Error, should be run with 2 parameters: <input file for sat solver> <output file with scheme description>\n");
		return 0;
	}

	freopen(argv[1], "r", stdin);

	scanf("%d %d %d", &n, &m, &outs);
	gates.resize(n + m + 1);
	values.resize(outs);
	for (int k = 0; k < outs; k++) {
		values[k].resize(1 << n);
		for (int i = 0; i < (1 << n); i++) {
			scanf("%d", &values[k][i]);	
		}
	}
	int tn, tm, touts;
	freopen(argv[2], "r", stdin);
	scanf("%d %d %d", &tn, &tm, &touts); 
	assert(n == tn);
	assert(m == tm);
	assert(outs == touts);
	out.resize(outs);
	for (int i = 0; i < outs; i++) {
		scanf("%d", &out[i]);
	}
	scanf("\n");
	for (int i = n + 1; i <= n + m; i++) {
		int num;
		char buf[100];
		gets(buf);
		for (int i = 0; i < strlen(buf); i++) {
			if (!('0' <= buf[i] && buf[i] <= '9')) {
				buf[i] = ' ';
			}
		}
		sscanf(buf, "%d", &num);
		sscanf(buf, "%d %d %d %d", &num, &gates[num].inputs[0], &gates[num].inputs[1], &gates[num].type);			
	}

	int errors = 0;
	vector <vector<int> > gatevalues(n + m + 1);
	for (int mask = 0; mask < (1 << n); mask++) {
		int curmask = mask;
		for (int i = 1; i <= n; i++) {
			gates[i].outvalue = curmask & 1;
			curmask >>= 1;
		}
		for (int i = n + 1; i <= n + m; i++) {
			gates[i].getoutvalue(gates[gates[i].inputs[0]].outvalue, gates[gates[i].inputs[1]].outvalue);
			gatevalues[i].push_back(gates[i].outvalue);
		}
		for (int k = 0; k < outs; k++) {
			if (values[k][mask] == 2) {
				continue;
			}
			if (values[k][mask] != gates[out[k]].outvalue) {
				errors++;
				fprintf(stderr, "Error on input %d on output %d, %d instead of %d\n", mask, k, gates[out[k]].outvalue, values[k][mask]);
				for (int i = 1; i <= n + m; i++) {
					fprintf(stderr, "%d ", gates[i].outvalue);
				}
				fprintf(stderr, "\n");
			}
		}
		outtable[mask].resize(n + m + 2);
		for (int i = 1; i <= n + m; i++) {
			outtable[mask][i] = gates[i].outvalue;
		}
/*		fprintf(stderr"Input: ");
		for (int i = 0; i < n; i++) {
			fprintf(stderr"%d ", (mask >> i) & 1);
		}
		fprintf(stderr"Output: ");
		for (int k = 0; k < outs; k++) {
			fprintf(stderr"%d ", gates[out[k]].outvalue); 
		}
		fprintf(stderr"\n");*/

	}

	printtextablev();
	printtextableh();


	cerr << "HERE" << endl;
	fprintf(stderr, "HERE %d %d\n", n, m);

	for (int i = n + 1; i <= n + m; i++) {
		fprintf(stderr, "Gate %d\n", i);
		for (int j = 0; j < gatevalues[i].size(); j++) {
			fprintf(stderr, "%d ", gatevalues[i][j]);
		}
		fprintf(stderr, "\n");
	}

	if (errors) {
		fprintf(stderr, "!!!BUG!!!\n");
	} else {
		fprintf(stderr, "OK!\n");
	}

	return 0;
}
